@@ -1,7 +1,16 @@
 use evdev::Key;
+use serde::{Deserialize, Deserializer};
 use std::error::Error;
 use std::str::FromStr;
 
+pub fn deserialize_key<'de, D>(deserializer: D) -> Result<Key, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    let key = String::deserialize(deserializer)?;
+    Ok(parse_key(&key).map_err(serde::de::Error::custom)?)
+}
+
 pub fn parse_key(input: &str) -> Result<Key, Box<dyn Error>> {
     // Everything is case-insensitive
     let name = input.to_uppercase();
